import { DataResponse } from "./pcore";

export declare class PConnect {
    getCaseInfo(): CaseInfoManager;
    getContextName(): string;
    getValue<T>(key: string): T;
    getDataObject(key?: string): ContainerItemData;
    getMetadata(): { children: Props[] };
    getFieldMetadata(fieldId: string): RawMetadata;
    getRawMetadata(): RawMetadata;
    getPageReference(): string;
    getActionsApi(): ActionsApi;
    getValidationApi(): ValidationApi;
    getChildren(): Props[];
    getComponentName(): string;
    getConfigProps(): ConfigProps;
    getStateProps(): any;
    getContainerManager(): ContainerManager;
    getContainerName(): string;
    isBoundToState(): boolean;
    isConditionExist(): boolean;
    populateAdditionalProps(props: any): void;
    resolveConfigProps(props: ConfigProps): any;
    createComponent(meta: PConnectMeta, dataSource: string | null, index: string | null, additionalProps: { pageReference?: string }): PComponent;
    getReferencedView(): PConnectMeta;
    getComponentConfig(): PConnectMetaConfig;
    resolveDatasourceReference(datapageReference: string): Datapage | string;
    setInheritedConfig(config: PConnectMetaConfig): void;
    getInheritedProps(): any;
    setInheritedProp(key: string, value: any): void;
    setReferenceList(list: any): void;
    /**
 * Returns the resolved props from the provided input props and registers them to configuration object which are used for component rendering.
 * When you call the getConfigProps API, the resolved properties are returned.
 *
 * @function registerAdditionalProps
 * @example
 * Example:
 * additionalMeta = {
 *  label: "@P .FirstName"
 * }
 * output:
 * {
 *   label: "Hello Name"
 * }
 * @param additionalMeta Props meant to be resolved.
 * @example <caption>getPConnect().registerAdditionalProps()</caption>
 * const props = getPConnect().registerAdditionalProps({ label: `@P .FirstName` });
 * the props variable will be an object with 'label' value is field value entered for FirstName field.
 * Also this prop ('label') is registered in component configuration for further usage in re-renders.
 * In this example, the API resolves the FirstName property, obtains the first name value, and registers it to label.
 *
 * @returns resolved props.
 * @public
 */
    registerAdditionalProps(additionalMeta?: object): any;

    /**
 * Returns the localized value for a string.
 *
 * @function getLocalizedValue
 * @example <caption>getPConnect().getLocalizedValue("First Name", "fields","WORK-!VIEW!CANDIDATEINFO")</caption>
 * If the localization store looks like below.
 *
 * {
 *     "WORK-!VIEW!CANDIDATEINFO" : {
 *          "fields" : {
 *              "First Name" : "Primero nome",
 *              "Last Name"  : "Ultimo nome"
 *           }
 *      }
 * }
 * The API returns "Primero nome" for a rule  "WORK-!VIEW!CANDIDATEINFO".
 *
 * @param rawString      raw string
 * @param localePath     locale path
 * @param localeRuleKey Rule Key in the localization store
 * @returns returns the localized value.
 */
    getLocalizedValue(rawString: string, localePath?: string, localeRuleKey?: string): string;
}

export interface PComponent {
    props: Props;
}

export interface PConnectMeta {
    type: string;
    config: PConnectMetaConfig;
}

export interface PConnectMetaConfig {
    name: string;
    type?: string;
    context?: string;
    visibility?: boolean;
    referenceList?: string;
    readOnly?: boolean;
    displayMode?: string;
}

export declare class Props {
    getPConnect(): PConnect;
}

export declare class CaseInfoManager {
    getActions(): string[];
    getActiveFlowActionID(): string;
    getAssignmentID(): string;
    getCaseStages(caseID: string): Stage[];
    getCaseTypeName(): string;
    getClassName(): string;
    getCurrentAssignmentViewName(): string;
    getID(): string;
    getKey(): string;
    getName(): string;
    getParentCaseInfo(): CaseInfo
    isPerform(): boolean;
    isReview(): boolean;
    c11nEnv: PConnect;
}

export interface ContainerConfig {
    type: string;
}

export declare class ContainerManager {
    initializeContainers(config: ContainerConfig): Promise<void>;
    addContainerItem(config: ContainerItem): Promise<void>;
}

export interface ContextData {
    caseInfo: CaseInfo,
    caseViewMode: string;
}

export interface Datapage {
    pxResults: any[];
}

export interface Page {
    pyRuleName: string;
    pyClassName: string;
    pxPageViewIcon: string;
    pyLabel: string;
}

export interface CaseType {
    pyClassName: string;
    pyFlowType: string;
    pxPageViewIcon: string;
    pyLabel: string;
}

export interface Task {
    operator: string;
    id: string;
    name: string;
    value: string;
    priority: number;
    classname: string;
    status: string;
    stepName: string;
}

export interface ContainerItemData {
    caseInfo: CaseInfo;
    referencedUsers: { [key: string]: { UserName: string } };
    nextAssignmentInfo: Assignment,
    context_data: ContextData;
    D_pyCountryCallingCodeList?: Datapage
}

export interface ContainerItem {
    semanticURL: string;
    caseViewMode?: string;
    data?: ContainerItemData;
    containerType?: string;
    resourceType?: string;
    context?: string;
    acName?: string;
    key?: string;
    flowName?: string;
    containerItemID?: string;
    target?: string
}
export interface Assignment {
    assigneeInfo?: User;
    instructions?: string;
    urgency?: string;
    processID?: string;
    name?: string;
    context: string;
    canPerform?: string;
    links: Links,
    ID: string;
    actions?: Action[];
}

export interface CaseAction {
    ID: string;
    name: string;
}

export interface SLA {
    goal: string;
    deadline: string;
}

export interface Stage {
    entryTime: string;
    name: string;
    links: Links;
    visited_status: string;
    ID: string;
    type: string;
    transitionType: string;
}

export interface ActionButton {
    jsAction: string;
    name: string;
    actionID: string;
}

export interface ActionButtons {
    main: ActionButton[];
    secondary: ActionButton[];
}

export interface NavigationStep {
    steps: NavigationStep[];
    allow_jump: boolean;
    name: string;
    actionID: string;
    visited_status: string;
    links?: Links,
    ID: string;
}

export interface Navigation {
    template: string;
    steps: NavigationStep[];
}

export interface CaseInfo {
    content: any;
    caseTypeID: string;
    owner: string;
    availableActions: AvailableAction[];
    associations: Associations;
    lastUpdatedBy: string;
    assignments: Assignment[];
    hasNewAttachments: boolean;
    sla: SLA;
    urgency: string;
    createTime: string;
    createdBy: string;
    name: string;
    stages: Stage[];
    ID: string;
    status: string;
    stageID: string;
    stageLabel: string;
    lastUpdateTime: string;
    isFirstAssignment: boolean;
    actionButtons: ActionButtons;
    navigation: Navigation;
    headers: { [key: string]: string };
}

export interface ViewConfig {
    context?: string;
    name?: string;
    label?: string;
    value?: string;
    caption?: string;
    acTertiary?: string;
}

export interface ItemView {
    config: ViewConfig;
    type: string;
}

export interface RoutingItem {
    view: ItemView;
    semanticURL: string;
    context: string;
}

export interface RoutingInfo {
    accessedOrder: string[];
    items: { [key: string]: RoutingItem };
    type: string;
}

export interface ConfigProps {
    routingInfo: RoutingInfo;
    template: string;
    title: string;
    acTertiary?: string;
}

export interface Work {
    containerName: string;
    flowType: string;
    caseInfo?: CaseInfo;
}

export declare class ValidationApi {
    validate(value: any): void;
}

/** 
 * Use the APIs in the ActionsApi class to handle actions in the Constellation
 * architecture.
 */
export declare class ActionsApi {
    /** 
     * Performs the approval action for the approval step configured in a case.
     * 
     * @param containerItemID - The ID of the container item that contains information about the
                        current case.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the approval action is
                successful.
     * ```
    const approveCasePromise = getPConnect().getActionsApi().approveCase("app/primary_1/workarea_1");
approveCasePromise.then(() => {
 // approve case success handling
}).catch(() => {
 // approve case failure handling
});
     ```
     */
    approveCase(containerItemID: string): Promise<void>;

    /** 
     * Cancels the current assignment.
     * 
     * @param containerItemID - The ID of the container item that contains information about the
                         current assignment.
    * 
    * @returns A Promise associated with the action.
    * 
    * @example In this example, the success callback is called if the current assignment is
                canceled.
    * ```
    const cancelAssignmentPromise = getPConnect().getActionsApi().cancelAssignment("app/primary_1/workarea_1"); 
cancelAssignmentPromise.then(() => {
// cancel assignment success handling
}).catch(() => {
// cancel assignment failure handling
});
    ```
    */
    cancelAssignment(containerItemID: string): Promise<void>;

    /** 
     * Cancels the current assignment in the create stage.
     * 
     * @param containerItemID - The ID of the container item that contains information about the
                        current assignment in the create stage.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the current assignment is canceled
                in the create stage.
     * ```
    const cancelCreateStageAssignmentPromise = getPConnect().getActionsApi().cancelCreateStageAssignment("app/modal_3"); 
cancelCreateStageAssignmentPromise.then(() => {
 // cancel create stage assignment success handling
}).catch(() => {
 // cancel create stage assignment failure handling
});
     ```
     */
    cancelCreateStageAssignment(containerItemID: string): Promise<void>;

    /** 
     * Cancels the creation of a data object whose create view is loaded in the specified
        container item.
     * 
     * @param containerItemID - The ID of the container item containing the create view of the data
                        object.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the creation of the data object is
                canceled.
     * ```
    const cancelCreateDataObjectPromise = getPConnect().getActionsApi().cancelDataObject("app/modal_3");
cancelCreateDataObjectPromise.then(() => {
 // cancel creation of data object success handling
}).catch(() => {
 // cancel creation of data object failure handling
});
     ```
     */
    cancelDataObject(containerItemID: string): Promise<void>;

    /** 
     * Cancels the local action that is associated with the container item ID of the current
        component.
     * 
     * @param containerItemID - The ID of the container item associated with the current
                        component.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the local action is canceled.
     * ```
    const cancelLocalAction = getPConnect().getActionsApi().cancelLocalAction("app/modal_3");
cancelLocalAction.then(() => {
 // cancel local action success handling
}).catch(() => {
 // cancel local action failure handling
});
     ```
     */
    cancelLocalAction(containerItemID: string): Promise<void>;

    /** 
     * Runs a change handler for the given context of the component through the Action
        Processor.
     * 
     * @param pConn - The PConnect object associated with the handler being run.
     * @param event - The Document Object Model (DOM) event object associated with the
                        change.
     * 
     * @returns Not applicable.
     * 
     * @example In this example, the API handles the change event triggered by the UI component.
     * ```
    getPConnect().getActionsApi().changeHandler(pConnObject, event);
     ```
     */
    changeHandler(pConn: PConnect, event: Event): void;

    /** 
     * Creates a data object whose create view is loaded in the specified container
        item.
     * 
     * @param containerItemID - The ID of the container item containing the create view of the data
                        object.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the data object is created.
     * ```
    const createDataObjectPromise = getPConnect().getActionsApi().createDataObject("app/modal_3");
createDataObjectPromise.then(() => {
 // create data object success handling
}).catch(() => {
 // create data object failure handling
});
     ```
     */
    createDataObject(containerItemID: string): Promise<void>;

    /** 
     * Creates a work object for a specified class and flow type and displays the work
        object in a target container.
     * 
     * @param className - Name of the case class for which the work object should be
                        created.
     * @param options - The JavaScript object that contains the properties required for
                        creating the work object. 
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the API creates a work object within the primary
                container with the OPB1HW-SpaceTra-Work-RequestApproval class and
                the pyStartCase flow type.
     * ```
    const options = {
     flowType: "pyStartCase",
     containerName: "primary",
     startingFields: {
         FirstName: "Adam",
         LastName: "Smith",
         Vehicle: {
             Make: "Honda",
             Model: "Accord"
         }
     }
 };

const createWorkPromise = getPConnect().getActionsApi().createWork("OPB1HW-SpaceTra-Work-RequestApproval", options); 
createWorkPromise.then(() => {
  // create work success handling
}).catch(() => {
  // create work failure handling
});
     ```
     */
    createWork(className: string, options: Work): Promise<void>;

    /** 
     * Deletes the case that is currently in the create stage.
     * 
     * @param containerItemID - The ID of the container item that contains information about the case
                        that is currently in the create stage.
     * @param ignoreCaseDeletion - The flag that determines if a case will be deleted in the create
                            stage.
                            Note: The default value is false.
                                Set ignoreCaseDeletion to
                                        true if the case should not be deleted in
                                    the create stage.
                                Set ignoreCaseDeletion to
                                        false if the case should be deleted in the
                                    create stage.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the case that is currently in the
                create stage is deleted.
     * ```
    const deleteCaseInCreateStagePromise = getPConnect().getActionsApi().deleteCaseInCreateStage("app/modal_1");
deleteCaseInCreateStagePromise.then(() => {
 // delete case in create stage success handling
}).catch(() => {
 // delete case in create stage success handling
});
     ```
     */
    deleteCaseInCreateStage(containerItemID: string, ignoreCaseDeletion?: boolean): Promise<void>;

    /** 
     * Deletes the specified data object.
     * 
     * @param className - The name of the class that contains the data object to be
                        deleted.
     * @param keys - The object that contains the unique identifier of the data object to be
                        deleted.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the data object is deleted.
     * ```
    getPConnect().getActionsApi().deleteDataObject('OEDUS1-ReactApp-Data-Person', { pyGUID: '3ba585e8-f3e2-4404-8a15-692992de53b4' }).then(() => {
 // deleteDataObject success handling
}).catch(() => {
 // deleteDataObject failure handling
});
     ```
     */
    deleteDataObject(className: string, keys: object): Promise<void>;

    /** 
     * Runs an eventHandler for the given context of the component through the Action
        Processor.
     * 
     * @param pConn - The PConnect object associated with the handler being
                        registered.
     * @param event - The Document Object Model (DOM) event object associated with the
                        event.
     * 
     * @returns Not applicable.
     * 
     * @example In this example, the API handles the event triggered by the UI component.
     * ```
    getPConnect().getActionsApi().eventHandler(pConnObject, event);
     ```
     */
    eventHandler(pConn: object, event: Event): void;

    /** 
     * Submits the assignment that is associated with the given case (case details are
        obtained from the metadata when the application loads) and the given flow container ID. 
     * 
     * @param containerItemID - The ID of the flow container that is associated with the assignment
                        that must be submitted.
     * @param options - The JavaScript object that contains optional properties that provide
                        additional information for submitting the assignment.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the assignment associated with the
                given container information is submitted successfully.
     * ```
    const finishAssignmentPromise = getPConnect().getActionsApi().finishAssignment("app/primary_1/workarea_1");
finishAssignmentPromise.then(() => {
 // finish assignment success handling
}).catch(() => {
 // finish assignment failure handling
});
     ```
     */
    finishAssignment(containerItemID: string, options?: object): Promise<void>;

    /** 
     * Displays the edit view of a data object that needs to be updated.
     * 
     * @param className - The name of the class that provides the type that the data object
                        should belong to.
     * @param keyProperties - The object containing a set of keys to identify the data
                        object.
     * @param options - The JavaScript object containing the property to render the edit view
                        in the modal.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the edit view of the data object
                is displayed.
     * ```
    getPConnect().getActionsApi().getDataObjectView('OZR2SN-ReactApp-Data-Person', {pyGUID: "61a8e531-afcb-41f7-bd72-9809e1a3cbe9"}).then(() => {
 // getDataObjectView success handling
}).catch(() => {
 // getDataObjectView failure handling
});
     ```
     */
    getDataObjectView(className: string, keyProperties: object, options?: object): Promise<void>;

    /** 
     * Obtains the assignment that contains the highest priority.
     * 
     * @returns An empty Promise.
     * 
     * @example In this example, the success callback is called if the assignment that contains the
                highest priority is obtained successfully.
     * ```
    const getNextWorkPromise = getPConnect().getActionsApi().getNextWork();
getNextWorkPromise.then(() => {
 // open Next Work success handling
}).catch(() => {
 // open Next Work failure handling
});
     ```
     */
    getNextWork(): Promise<void>;

    /** 
     * Obtains the list of the recently accessed assignments.
     * 
     * @param maxResultsToFetch - The maximum number of results to be obtained from the server.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the specified list of the recently
                accessed assignments is obtained successfully.
     * ```
    const recentsPromise = getPConnect().getActionsApi().getRecents(15);
recentsPromise.then((data) => {
 // success handling
}).catch(() => {
 // failure handling
});
     ```
     */
    getRecents(maxResultsToFetch: number): Promise<DataResponse<Assignment>>;

    /** 
     * Performs the ignore duplicate action for the search duplicate cases automation step
        configured in a case.
     * 
     * @param containerItemID - The ID of the container item that provides the current context of the
                        case.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the API performs the ignore duplicate action for the search
                duplicate cases automation step configured in the
                    app/primary_1/workarea_1 container.
     * ```
    const ignoreDuplicateCasePromise = getPConnect().getActionsApi().ignoreDuplicateCase("app/primary_1/workarea_1");
ignoreDuplicateCasePromise.then(() => {
 // ignore duplicate case success handling
}).catch(() => {
 // ignore duplicate case failure handling
});
// The success callback is called if the ignore duplicate case action is successful.
     ```
     */
    ignoreDuplicateCase(containerItemID: string): Promise<void>;

    /** 
     * Invokes a REST API using the fetch function of a service broker.
     * 
     * @param url - The URL for the
                        request.Example:/prweb/api/v1/messages?filterFor=DATA-PORTAL $SpaceTra
/prweb/api/v1/data/D_pxOperatorDetails?OperatorId=user%40c11ngallery.com,
     * @param opts - The HTTP methods that are used in the request body to fetch the REST
                        API.
     * 
     * @returns A Promise for the pre-processed response.
     * 
     * @example In this example, the API invokes the URL with the options provided, and returns a
                response.
     * ```
    invokePromise = getPConnect().getActionsApi().invoke(url, options); 
invokePromise.then(() => {
  // invoke success handling
}).catch(() => {
  // invoke failure handling
});
     ```
     */
    invoke<T>(url: string, opts?: { method: string, body: object }): Promise<DataResponse<T>>;

    /** 
     * Loads a view with data from server based on the context of a case.
     * 
     * @param caseId - Unique identifier of the case associated with a context.
     * @param viewName - Name of the view which must be loaded with data.
     * @param config - Object that passes the context and container information.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the API loads the pyReview view with data from a
                server based on the context of the case whose ID is ON8TTL-C11NGALL-WORK C-7001. 
     * ```
    const loadViewPromise = getPConnect().getActionsApi().loadView("ON8TTL-C11NGALL-WORK C-7001","pyReview");
loadViewPromise.then(() => {
  // load view success handling
}).catch(() => {
  // load view failure handling
});
     ```
     */
    loadView(caseId: string, viewName: string, config?: object): Promise<void>;

    /** 
     * Navigates to a specific step in the context of a container.
     * 
     * @param stepID - The ID of the step to be navigated to. Use the value
                            previous to go to the previous step.
     * @param containerItemID - The ID of the container item.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the navigate action is
                successful.
     * ```
    navigateToStepPromise = getPConnect().getActionsApi().navigateToStep("Step1", "app/primary_1/workarea_1");
navigateToStepPromise.then(() = > {
 // navigate to step success handling
}).catch(() => {
 // navigate to step failure handling
});
     ```
     */
    navigateToStep(stepID: string, containerItemID: string): Promise<void>;

    /** 
     * Opens an assignment and stores it in a target container.
     * 
     * @param assignmentID - The ID of the assignment to be
                        opened.Example:"ASSIGN-WORKLIST OPB1HW-SPACETRA-WORK RA-43001!REQUEST_FLOW_0",
     * @param className - The name of the case class associated with the assignment.
                            Example:"OPB1HW-SpaceTra-Work-RequestApproval",
     * @param options - The JavaScript object containing a property of the target
                        container.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the assignment is opened
                successfully.
     * ```
    const openAssignmentPromise = getPConnect().getActionsApi().openAssignment(
     "ASSIGN-WORKLIST OPB1HW-SPACETRA-WORK RA-43001!REQUEST_FLOW_0",
     "OPB1HW-SpaceTra-Work-RequestApproval",
     { containerName: "primary" }
 );
openAssignmentPromise.then(() => {
 // open assignment success handling
}).catch(() => {
 // open assignment failure handling
});
     ```
     */
    openAssignment(assignmentID: string, className: string, options?: Work): Promise<void>;

    /** 
     * Opens the configured local action.
     * 
     * @param actionID - The unique identifier of the local action.
     * @param options - The JavaScript object containing the properties for opening the local
                        action.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the local action is launched
                successfully.
     * ```
    const localActionPromise = getPConnect().getActionsApi().openLocalAction("EditDetails", { type : "Case" });
localActionPromise.then(() => {
 // local action success handling
}).catch(() => {
 // local action failure handling
});
     ```
     */
    openLocalAction(actionID: object, options: LocalAction): Promise<void>;

    /** 
     * Opens the configured process action.
     * 
     * @param actionID - The ID of the process action.
     * @param options - The JSON object containing the container target in which the process
                        action is displayed.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the process action is launched
                successfully.
     * ```
    const processActionPromise = getPConnect().getActionsApi().openProcessAction("VendorAddition_Flow", { type: "Case" });
processActionPromise.then(() => {
 // open process action success handling
}).catch(() => {
 // open process action failure handling
});
     ```
     */
    openProcessAction(actionID: object, options: ProcessAction): Promise<void>;

    /** 
     * Opens a work object associated with a work ID.
     * 
     * @param workID - The work object to be opened.
     * @param className - The name of the case class associated with the work object.
     * @param options - The JavaScript object that contains the properties required for opening
                        the work object. 
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the work object is opened
                successfully.
     * ```
    const openWorkPromise = getPConnect().getActionsApi().openWorkByHandle("OPB1HW-SPACETRA-WORK RA-10001", "OPB1HW-SpaceTra-Work-RequestApproval");
openWorkPromise.then(() => {
 // open work by handle success handling
}).catch(() => {
 // open work by handle failure handling
});
     ```
     */
    openWorkByHandle(workID: string, className: string, options?: { targetContainer: string }): Promise<void>;

    /** 
     * Performs the reject action for the approval step configured in a case.
     * 
     * @param containerItemID - The ID of the flow container.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the reject action is
                successful.
     * ```
    const rejectCasePromise = getPConnect().getActionsApi().rejectCase("app/primary_1/workarea_1");
rejectCasePromise.then(() => {
 // reject case success handling
}).catch(() => {
 // reject case failure handling
});
     ```
     */
    rejectCase(containerItemID: string): Promise<void>;

    /** 
     * Performs the resolve duplicate action for the search duplicate cases automation step
        configured in a case.
     * 
     * @param containerItemID - The ID of the container item that provides the current context of the
                        case.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the API performs the resolve duplicate action for the search
                duplicate cases automation step configured in the
                    app/primary_1/workarea_1 container.
     * ```
    const resolveDuplicateCasePromise = getPConnect().getActionsApi().resolveDuplicateCase("app/primary_1/workarea_1");
resolveDuplicateCasePromise.then(() => {
 // resolve duplicate case success handling
}).catch(() => {
 // resolve duplicate case failure handling
});
// The success callback is called if the resolve duplicate case action is successful.
     ```
     */
    resolveDuplicateCase(containerItemID: string): Promise<void>;

    /** 
     * Saves the current work object opened in the modal dialog.
     * 
     * @param containerItemID - The ID of the container item that contains information about the
                        current work object.Example: "app/modal_1"
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the case is saved
                successfully.
     * ```
    const saveAndClosePromise = getPConnect().getActionsApi().saveAndClose("app/modal_1"); 
saveAndClosePromise.then(() => {
 // save and close success handling
}).catch(() => {
 // save and close failure handling
});
     ```
     */
    saveAndClose(containerItemID: string): Promise<void>;

    /** 
     * Saves the opened assignment.
     * 
     * @param containerItemID - The ID of the container item that contains information about the opened
                            assignment.Example: "app/modal_1"
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the opened assignment is saved
                successfully.
     * ```
    const saveAssignmentPromise = getPConnect().getActionsApi().saveAssignment("app/modal_1");
saveAssignmentPromise.then(() => {
 // saveAssignment success handling
}).catch(() => {
 // saveAssignment failure handling
});
     ```
     */
    saveAssignment(containerItemID: string): Promise<void>;

    /** 
     * Displays a preview of a case item inside the preview panel.
     * 
     * @param pzInsKey - The unique identifier of the case item.Example:
                                OPB1HW-SPACETRA-WORK 20RA-2,
     * @param configObj - The name of the case item's class.
     * 
     * @returns Not applicable.
     * 
     * @example In this example, the API displays the preview of a case item (identified by a unique
                key called pzInsKey) inside the preview panel.
     * ```
    getPConnect().getActionsApi().showCasePreview(pzInsKey, configObj);
     ```
     */
    showCasePreview(pzInsKey: string, configObj?: object): void;

    /** 
     * Displays the data of a page based on the data context.
     * 
     * @param pageName - The name of the view in which the data must be displayed.
     * @param dataContext - The name of the page type data page whose data must be
                        displayed.
     * @param dataContextParameters - The parameters associated with the data page.
     * @param options - The JavaScript object containing the properties to display the
                        data.
     * 
     * @returns A promise associated with the action.
     * 
     * @example In this example, the API displays the data of the EmployeeDetails
                page based on the D_EmployeeDetails data context.
     * ```
    const showDataPromise = getPConnect().getActionsApi().showData("EmployeeDetails", "D_EmployeeDetails", {pyGUID: "0759409f-4146-439c-aa25-57d4f495fee5"});
showDataPromise.then(() => {
  // show data success handling
}).catch(() => {
  // show data failure handling
});
     ```
     */
    showData(pageName: string, dataContext: string, dataContextParameters: object, options?: DataOptions): Promise<void>;

    /** 
     * Displays the create view of a data object to be created.
     * 
     * @param className - The name of the class that provides the type that the data object
                        should belong to.
     * @param viewName - The name of the create view that is to be displayed for the data
                        object.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the create view is displayed.
     * ```
    getPConnect().getActionsApi().showDataObjectCreateView('OZR2SN-ReactApp-Data-Person', 'Create').then(() => {
// showDataObjectCreateView success handling
}).catch(() => {
// showDataObjectCreateView failure handling
});
     ```
     */
    showDataObjectCreateView(className: string, viewName?: string): Promise<void>;

    /** 
     * Displays a preview of a data item in the preview panel.
     * 
     * @param dataContext - The name of the data page whose data must be displayed.
     * @param dataContextParameters - The parameters associated with the data page.
     * @param options - The JavaScript object containing the properties to display the
                            data.Note: The properties of this parameter are generated by the system
                            and cannot be provided by the user.
     * 
     * @returns A promise associated with the action.
     * 
     * @example In this example, the API displays the preview of a data item (identified by
                    dataContext and the dataContextParameters)
                inside the preview panel with the extra configuration object provided.
     * ```
    getPConnect().getActionsApi().showDataPreview(dataContext, dataContextParameters, options);
     ```
     */
    showDataPreview(dataContext: string, dataContextParameters: string, options?: object): Promise<void>;

    /** 
     * Displays the page of a class.
     * 
     * @param pageName - The name of the page to be displayed.Example: pageName =
                                "pyHome",
     * @param className - The name of the class to which the page belongs.Example:
                                className: "Data-Portal",
     * @param options - The JavaScript object containing the properties to display the
                        page.
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the API obtains and displays the view metadata and class name of a
                page.
     * ```
    const showPagePromise = getPConnect().getActionsApi().showPage("pyHome", "Data-Portal");
showPagePromise.then(() => {
  // show page success handling
}).catch(() => {
  // show page failure handling
});
     ```
     */
    showPage(pageName: string, className: string, options?: DataOptions): Promise<void>;

    /** 
     * Runs validations on a specified control and triggers the
            FIELD_CHANGE event.
     * 
     * @param propName - The name of the control on which the FIELD_CHANGE
                        event must be invoked.
     * @param value - The value of the control that will be validated and passed when
                        invoking the FIELD_CHANGE event.
     * 
     * @returns Not applicable.
     * 
     * @example In this example, the API validates and invokes the FIELD_CHANGE
                event as triggered by the specified control.
     * ```
    getPConnect().getActionsApi().triggerFieldChange(propName, value);
     ```
     */
    triggerFieldChange(propName: string, value: any): void;

    /** 
     * Updates a data object whose edit view is loaded in the specified container
        item.
     * 
     * @param containerItemID - The ID of the container item containing the edit view of the data
                        object.
     * @param keys - The keys of the data record to be updated.
                    Example:`{"pyGUID":"b1715d32-7d7b-4d41-97ce-ced87b7c8fb0"}`
     * 
     * @returns A Promise associated with the action.
     * 
     * @example In this example, the success callback is called if the data object is updated.
     * ```
    const updateDataObjectPromise = getPConnect().getActionsApi().updateDataObject("app/modal_3", '{"pyGUID":"61a8e531-afcb-41f7-bd72-9809e1a3cbe9"}');
updateDataObjectPromise.then(() => {
   updateDataObject success handling
}).catch(() => {
   updateDataObject failure handling
});
     ```
     */
    updateDataObject(containerItemID: string, keys: object): Promise<void>;

    /** 
     * Dispatches the SET_PROPERTY event on a specified control to store a
        specified value in the Redux Store.
     * 
     * @param propName - The name of the control whose value must be stored in the Redux
                        Store.
     * @param value - The value of the control to be stored in the Redux Store.
     * @param options - The JavaScript object containing the properties to enhance the
                        functionality of this API.
     * 
     * @returns Not applicable.
     * 
     * @example In this example, the API stores the value passed in the Redux Store as triggered by
                the control.
     * ```
    getPConnect().getActionsApi().updateFieldValue(propName, value);
     ```
     */
    updateFieldValue(propName: string, value: any, options?: UpdateOptions): void;

    fillFormWithAI(containerItemId: string): void;
}

export interface Link {
    rel: string;
    href: string;
    title: string;
    type: string;
}

export interface Links {
    [type: string]: Link
}

export interface AvailableAction {
    name: string;
    ID: string;
    type: string;
    links: Links;
}

export interface Associations {
    follows: boolean;
}

export interface Action {
    name: string;
    links: Links;
    ID: string;
    type: string;
}

export interface UpdateOptions {
    /**
     * 	The flag that removes the entry of the property from the changedPropertyList.
Note: The default value is false. Set this property to true to remove the entry of the property from the changedPropertyList.
     */
    removePropertyFromChangedList?: boolean;

    /**
     * 	The flag that decides if the values within an array or object must be deep merged to the Redux Store.
Note: The default value is true. Set this property to true to deep merge the values to the Redux Store.
     */
    isArrayDeepMerge?: boolean;

    /**
     * 	The flag that decides if the error related data for a specific entry within a list should be deleted.
Note: The default value is false. Set this property to true to delete all error related data for a specific entry within a list.
     */
    isListEntry?: boolean;
}

export interface DataOptions {
    /**
     * The name of the container that displays the data.
     */
    containerName?: string;
    /**
     * The flag that determines if the semantic URL evaluation logic must be skipped.
     * Note: The default value is false.
     * If skipSemanticUrl is false, the semantic URL evaluation logic will not be skipped.
     * If skipSemanticUrl is true, the semantic URL evaluation logic will be skipped.
     */
    skipSemanticUrl?: boolean;
}

export interface LocalAction {
    /**
     * The unique ID of the case. If this ID is not provided, it will be obtained using the current context.
Note: This is applicable for Case/Stage wide local action.
     */
    caseID?: string;
    /**
     * The unique ID of the assignment. If this ID is not provided, it will be obtained using the current context.
Note: This is applicable for local action of type Assignment.
     */
    assignKey?: string;
    /**
     * The title of the local action. This is used as the modal header title when the local action is displayed in the modal dialog.
     */
    name?: string;
    /**
     * The type of the local action. The values are:
     * - Assignment - Used for an assignment level local action.
     * Example:
```{ 
    "name": "Transfer assignment",
    "type": "Assignment",
    "containerName": "modal"
}```
    * - Case - Used for a case-wide local action.
    * Example:
```{ 
    "name": "Change stage", 
    "type": "Case", 
    "containerName": "workarea" 
}```
    * - Express - Used to call the case-wide local action from a different page context other than the case page.
    * Example:
```{
    "name": "Edit details",
    "type": "express",
    "containerName": "modal",
    "assignKey": "ASSIGN-WORKLIST ON8TTL-C11NGALL-WORK D-206014!VEHICLEDETAILS_FLOW",
    "caseID": "ON8TTL-C11NGALL-WORK D-206014"
}```
    * - Stage - Used for a stage-wide local action.
    * Example:
```{ 
    "name": "Edit Details", 
    "type": "stage", 
    "containerName": "modal" 
}```
     */
    type: string;
    /**
     * The name of the container that is used to launch the local action in a modal or a work area.
     */
    containerName: string;
}

export interface ProcessAction {
    /**
     * The unique ID of the case. If this ID is not provided, it will be obtained using the current context.
     */
    caseID: string;
    /**
     * The title of the process action. This is used as the modal header title when the process action is displayed in the modal dialog.
     */
    actionTitle?: string;
    /**
     * 	The type of the process action. The values are Case or Stage.
     * Example:
```{caseID: "", actionTitle: "Vendor Addition", type: "Case" / "Stage"}```
     */
    type: string;
}

export interface User {
    name: string;
    ID: string;
    type: string;
}

export interface RawMetadata {
    children: ItemView[];
    config: ConfigMetadata;
    type: string;
    name: string;
}

export interface ConfigMetadata {
    datasource: ConfigMetadataDatasource;
    referenceList: string;
    value: string;
    presets: Preset[];
}

export interface Preset {
    name: string;
    template: string;
    config: Record<string, unknown>;
    label: string;
    id: string;
    children: RawMetadata[];
}

export interface ConfigMetadataDatasource {
    fields: DataSourceFields;
}

export interface DataSourceFields {
    value: string;
    key: string;
    text: string;
}