import { ContainerItem, ItemView, PComponent, Props } from './pconnect';

export declare class PCore<T> {
  getAttachmentUtils(): AttachmentUtils;
  getDataApiUtils(): DataApiUtils;
  getContainerUtils(): ContainerUtils;
  getEnvironmentInfo(): EnvironmentInfo;
  getAuthUtils(): AuthUtils;
  getStateUtils(): StateUtils;
  getEvents(): Events;
  getPubSubUtils(): PubSubUtils;
  getConstants(): PCoreConstants;
  getDebugger(): Debugger;
  checkIfSemanticURL(): boolean;
  getStore(): PStore<T>;
  getContainerItems(): ContainerItems;
  createPConnect(config: PConnectConfig): Props;
  setBehaviorOverride(key: string, value: any): void;
  onPCoreReady(callback: (root: PComponent, target: string) => void): void;
  getMashupApi(): MashupApi;
  registerComponentCreator(
    creator: (config: Props, additionalProps: any) => PComponent
  ): void;
  isDeepEqual(oldValue: any, newValue: any): boolean;
  getMessageManager(): MessageManager;
  getRestClient(): RestClient;
  getStoreValue<T>(propKey: string, pageKey: string, context: string): T;

    /**
   * Obtains an entry point to the ActionsSequencer object that contains APIs to sequence  different type of actions in the Constellation Infrastructure.
   * To view the APIs in the ActionsSequencer class, see {@link ActionsSequencer|APIs in the ActionsSequencer class}
   *
   * @example <caption>In this example , the API returns the ActionsSequencer object containing the utility APIs</caption>
   * PCore.getActionsSequencer();
   *
   * @returns The ActionsSequencer Object
   * @function
   */

    getActionsSequencer(): ActionsSequencer;
}

/**
 * Provides the basic APIs to sequence different type of actions in the Constellation infrastructure.
 *
 * Actions are classified into 2 types blocking actions and deferred actions.
 *
 * For example: In an assignment, File Attachment is blocking action while finishAssignment is deferred action.
 * By using these apis, it is ensured that a deferred action is always fired when all blocking actions are completed.
 *
 */
export declare class ActionsSequencer {
  MISSING_CONTEXT_PARAM: string;
  INVALID_CONTEXT_DETAILS: string;
  INVALID_CONTEXT_OPERATION: string;
  /**
   * Dispatches queued actions on completion of currently running  action.Called from deRegisterBlockingAction,handleDeferredActionCompletion
   * @param context
   * For example: "app/modal_1"
   * @param contextDataObject data object for respective context fetched from redux
   *
   * @function
   * @private
   */
  schedulePendingActionOnCompletion(context: string, contextDataObject: {
      context_data: {
          blocking_action_counter: number;
          deferred_action_queue: any;
          is_deferred_action_in_progress: boolean;
      };
  }): void;
  /**
   * executes or queues deferred actions in the context of a container
   * @example <caption>In this example, the API tries to execute or queue the specified deferred action in the context of the 'app/primary_1' container and returns a promise containing the status of the deferred action</caption>
   * PCore.getActionsSequencer().executeOrQueueDeferredAction({
   * type: theType
   * payload: {
   *  context: app/primary_1,
   * //include other payload properties
   * }
   * }).then(successCallback).catch(failureCallback);
   * @param actionPayload The deferred action that must be executed or queued
   * @returns A promise associated with the action
   * <ul>
   * <li> The promise resolves successfully when the deferred action has been either executed or queued.
   * <li> The promise rejects with an error message when the deferred action has been neither executed nor queued.
   * </ul>
   *
   * @function
   */
  executeOrQueueDeferredAction(actionPayload: Action): Promise<void>;
  /**
   * signals the completion of execution of the ongoing deferred action and schedules the next deferred action in queue for execution
   * @example <caption>In this example, the API tries to signal the completion of execution of the ongoing deferred action and returns a promise associated with the action.</caption>
   * PCore.getActionsSequencer().handleDeferredActionCompletion("app/primary_1").then(successCallback).catch(failureCallback);
   * @param context  The name of the context where the API is being called
   * @returns A promise associated with the action
   * Note:
   * <ul>
   * <li> The promise resolves successfully when the execution of the deferred action has been completed.
   * <li> The promise rejects with an error message when the execution of the deferred action has not been completed.
   * </ul>
   *
   * @function
   */
  handleDeferredActionCompletion(context: string): Promise<void>;
  /**
   * Registers an ongoing blocking action in the context of a container
   * @example <caption>In this example, the API tries to register the ongoing blocking action in the context of the app/primary_1 container and returns a promise containing the registration status</caption>
   * PCore.getActionsSequencer().registerBlockingAction("app/primary_1").then(successCallback).catch(failureCallback);
   * @param context The name of the context where the APi si being called.
   * @returns A promise containing the registration status  of the ongoing blocking action
   * @function
   * @public
   */
  registerBlockingAction(context: string): Promise<void>;
  /**
   * De-registers an ongoing  blocking action in the context of a container when the blocking action is completed
   * @example <caption>In this example , the API tries to de-register the ongoing blocking action in the context of the app/primary_1 container and returns a promise containing the de-registration status</caption>
   * PCore.getActionsSequencer().deRegisterBlockingAction("app/primary_1").then(successCallback).catch(failureCallback);
   * @param context The name of the context where the API is being called.
   * @returns A promise containing the de-registration status  of the ongoing blocking action
   * @function
   * @public
   */
  deRegisterBlockingAction(context: string): Promise<void>;
  /**
   * cancels queued deferred actions when an error occurs
   * @example <caption>In this example, the API cancels queued deferred actions when an error occurs.</caption>
   * PCore.getActionsSequencer().cancelDeferredActionsOnError("app/primary_1");
   * @param context The name of the context where the API is being called.
   * @function
   * @public
   */
  cancelDeferredActionsOnError(context: string): void;
}

export declare class StateUtils {
  updateState(context: string, key: string, value: any, options: UpdateStateOptions);
}

export interface UpdateStateOptions {
  pageReference: string;
  isArrayDeepMerge: boolean;
}

export declare class RestClient {
  getHeaderProcessor(): HeaderProcessor;
}

export declare class HeaderProcessor {
  registerHeader(key: string, value: string): void;
}

/**
 * Provides APIs to access state-full properties like properties from pxRequestor, OperatorID and Application pages
 */
export declare class EnvironmentInfo {
  environmentInfoObject: EnvironmentInfoObj | null;
  isPortalLoaded: boolean;
  pageName: string | null;
  KeyMapping: any;
  isAssignmentHeaderHidden: boolean;
  coexistenceMeta: {
    appType: string;
    isCoexistence: boolean;
  };
  brandingInfo: any;
  themeOverrides: ConfigurationType['theme'];
  locale?: string;
  private cookieComplianceMethod;
  private showPerformanceDashboard;
  private isExtraCallRequiredForDefaultHome;
  constructor();
  setEnvironmentInfo(environmentInfoObject: EnvironmentInfoObj): void;
  /**
   * Returns operator user name
   *
   * @example <caption>PCore.getEnvironmentInfo().getOperatorName()</caption>
   * const operatorName = PCore.getEnvironmentInfo().getOperatorName();
   *
   * @returns user name
   */
  getOperatorName(): string | undefined;
  /**
   * Returns operator work group
   *
   * @example <caption>PCore.getEnvironmentInfo().getOperatorWorkGroup()</caption>
   * const operatorWorkGroup = PCore.getEnvironmentInfo().getOperatorWorkGroup();
   *
   * @returns work group name
   */
  getOperatorWorkGroup(): string | undefined;
  /**
   * Returns operator identifier
   *
   * @example <caption>PCore.getEnvironmentInfo().getOperatorIdentifier()</caption>
   * const operatorIdentifier = PCore.getEnvironmentInfo().getOperatorIdentifier();
   *
   * @returns identifier
   */
  getOperatorIdentifier(): string | undefined;
  /**
   * Returns operator image insKey
   *
   * @example <caption>PCore.getEnvironmentInfo().getOperatorImageInsKey()</caption>
   * const operatorImgInsKey = PCore.getEnvironmentInfo().getOperatorImageInsKey();
   *
   * @returns image insKey
   */
  getOperatorImageInsKey(): string | undefined;
  /**
   * Returns application label
   *
   * @example <caption>PCore.getEnvironmentInfo().getApplicationLabel()</caption>
   * const applicationLabel = PCore.getEnvironmentInfo().getApplicationLabel();
   *
   * @returns application label
   */
  getApplicationLabel(): string | undefined;
  getPortalName(): string;
  /**
   * Returns application name
   *
   * @example <caption>PCore.getEnvironmentInfo().getApplicationName()</caption>
   * const applicationLabel = PCore.getEnvironmentInfo().getApplicationName();
   *
   * @returns application name
   */
  getApplicationName(): string | undefined;
  /**
   * Returns useLocale defined
   *
   * @example <caption>PCore.getEnvironmentInfo().getUseLocale()</caption>
   * const useLocale = PCore.getEnvironmentInfo().getUseLocale();
   *
   * @returns useLocale
   */
  getUseLocale(): string | undefined;
  /**
   * Return Access group
   *
   * @example <caption>PCore.getEnvironmentInfo().getAccessGroup()</caption>
   * const accessGroup = PCore.getEnvironmentInfo().getAccessGroup();
   */
  getAccessGroup(): string | undefined;
  /**
   * Returns timeZone defined
   *
   * @example <caption>PCore.getEnvironmentInfo().getTimeZone()</caption>
   * const timeZone = PCore.getEnvironmentInfo().getTimeZone();
   *
   * @returns timezone
   */
  getTimeZone(): string | undefined;
  /**
   * Returns theme overrides defined
   *
   * @example <caption>PCore.getEnvironmentInfo().getTheme()</caption>
   * const theme = PCore.getEnvironmentInfo().getTheme();
   *
   * @returns theme
   */
  getTheme(): any;
  /**
   * updates theme overrides defined
   * @param theme - object with overrides to the theme definition
   * @example <caption>PCore.getEnvironmentInfo().setTheme(theme)</caption>
   * PCore.getEnvironmentInfo().setTheme(theme);
   */
  setTheme(theme: ConfigurationType['theme']): any;
  /**
   * Returns theme branding info defined
   *
   * @example <caption>PCore.getEnvironmentInfo().getBrandingInfo()</caption>
   * const theme = PCore.getEnvironmentInfo().getBrandingInfo();
   *
   * @returns theme branding info
   */
  private getBrandingInfo;
  /**
   * updates theme branding info defined
   * @param brandingInfo - object with overrides to the theme branding info
   * @example <caption>PCore.getEnvironmentInfo().setBrandingInfo(brandingInfo)</caption>
   * PCore.getEnvironmentInfo().setBrandingInfo(brandingInfo);
   */
  setBrandingInfo(brandingInfo: object): any;
  /**
   * Returns cookie compliance/privacy compliance method value defined on currently rendered portal.
   *
   * @example getCookieComplianceMethod()
   * Below api, returns 'default' as privacy compliance method for current rendered portal
   * ```
   * const cookieComplianceMethod = PCore.getEnvironmentInfo().getCookieComplianceMethod();
   * ```
   */
  getCookieComplianceMethod(): CookieComplianceMethod;
  /**
   * Sets cookie compliance/privacy compliance method value for current portal.
   * @param cookieComplianceMethod - selected privacy compliance method
   *
   * @example setCookieComplianceMethod()
   * Below api, sets default privacy compliance method value for current portal
   * ```
   * PCore.getEnvironmentInfo().setCookieComplianceMethod('default');
   * ```
   */
  setCookieComplianceMethod(cookieComplianceMethod: CookieComplianceMethod): void;
  /**
   * Use this API to change user's locale info on portal
   * to load localized application content.
   * This API must be called when portal is being loaded.
   * This API only changes locale in the portal but does not user's defined locale.
   * @param locale - User locale info that will be set on portal Example: de_DE, en_GB
   * @example <caption>In this example API sets user's locale to de_DE when portal is loading</caption>
   * PCore.getEnvironmentInfo().setLocale('de_DE')
   */
  setLocale(locale: string): void;
  getLocale: () => string | undefined;
  /**
   * Returns array containing all global environmental keys
   *
   * @example <caption>PCore.getEnvironmentInfo().getEnvironmentKeys()</caption>
   * const environmentKeys = PCore.getEnvironmentInfo().getEnvironmentKeys();
   *
   * @returns array containing all global environmental keys
   */
  getEnvironmentKeys(): string[];
  /**
   *This API obtains the system configured size limit in MB for attachments.
   *If the size limit is not configured in the system, then the default value of the size limit is 5.
   *
   * @returns the size limit.
   *
   * @example In this example, the API returns the system configured size limit for attachments.
   *
   * const maxAttachmentSize = PCore.getEnvironmentInfo().getMaxAttachmentSize();
   *
   */
  getMaxAttachmentSize(): string | undefined;
  /**
   *This API obtains the datapage to get the  operators list.
   *
   * @returns the operator datapage name.
   *
   * @example In this example, the API returns the datapage name to get the  operators list.
   *
   * const defaultOperatorDP = PCore.getEnvironmentInfo().getDefaultOperatorDP();
   *
   */
  getDefaultOperatorDP(): string | undefined;
  /**
   *This API obtains the datapage to get the  document url list.
   *
   * @returns the document datapage name.
   *
   * @example In this example, the API returns the datapage name to get the  document url list.
   *
   * const documentUrlDP = PCore.getEnvironmentInfo().getDocumentUrlDP();
   * @private
   */
  getDocumentUrlDP(): string | undefined;
  /**
   *This API obtains the datapage to get the  case instance list.
   *
   * @returns the case instance datapage name.
   *
   * @example In this example, the API returns the datapage name to get the case instance list.
   *
   * const caseInstanceListDP = PCore.getEnvironmentInfo().getCaseInstanceListDP();
   *
   */
  getCaseInstanceListDP(): string | undefined;
  /**
   * Returns the base locale
   *
   * @example <caption>PCore.getEnvironmentInfo().getBaseLocale()</caption>
   * const applicationLabel = PCore.getEnvironmentInfo().getBaseLocale();
   * //Returns en_EN if the base locale is set to en_EN during application creation.
   * @returns application name
   * @ignore
   */
  getBaseLocale(): string;
  /**
   * Description: Constellation UI can be rendered in three different modes
   * 1. FULL_PORTAL - which includes portal rendering of the constellation
   * 2. EMBED - It is a mashup scenario, where cases or views will be rendered
   * 3. HYBRID - Constellation redered in the legacy sections or harnesses
   * 4. PREVIEW - Constellation rendered during authoring time
   * This API returns the rederingMode value, which can be any value from the above four
   * @example <caption>PCore.getEnvironmentInfo().getRenderingMode()</caption>
   * const renderingMode = PCore.getEnvironmentInfo().getRenderingMode();
   * @returns renderingMode
   */
  getRenderingMode(): string | undefined;
  /**
   * Returns a csv list of built-on app names
   *
   * @example <caption>PCore.getEnvironmentInfo().getBuiltOnAppNames()</caption>
   * const builtOnAppNames = PCore.getEnvironmentInfo().getBuiltOnAppNames();
   *
   * @returns user name
   * @ignore
   */
  getBuiltOnAppNames(): string | undefined;
  /**
   * This API set the previewMode to true
   * @example <caption>PCore.getEnvironmentInfo().setPreviewMode()</caption>
   * @private
   */
  setPreviewMode(): void;
  /**
   * This API sets the defaultCasePage value
   * @example <caption>PCore.getEnvironmentInfo().setDefaultCasePage("pyEmbedAssignment")</caption>
   * @ignore
   */
  setDefaultCasePage(pageName: string): void;
  /**
   * This API gets the defaultCasePage value
   * @example <caption>PCore.getEnvironmentInfo().getDefaultCasePage()</caption>
   * @returns pageName value which has been already set
   * @ignore
   */
  getDefaultCasePage(): string | null;
  /**
   * This API sets the DisableAssignmentHeader value
   * @example <caption>PCore.getEnvironmentInfo().disableAssignmentHeader(true)</caption>
   * @ignore
   */
  disableAssignmentHeader(value: boolean): void;
  /**
   * This API gets the DisableAssignmentHeader value
   * @example <caption>PCore.getEnvironmentInfo().isAssignmentHeaderDisabled()</caption>
   * @returns disableAssignmentHeader value which has been already set
   */
  private isAssignmentHeaderDisabled;
  /**
   * This API gets the name of default portal assigned to an operator
   * @example <caption>if Operator has WebPortal as their default portal, PCore.getEnvironmentInfo().getDefaultPortal() will return "WebPortal"</caption>
   * @returns default portal name set to the operator
   */
  getDefaultPortal(): string | undefined;
  /**
   * This API is used to update additional feature info to environmentInfo Object
   * @example <caption>, PCore.getEnvironmentInfo().updateFeatureMap() will return feature updated object</caption>
   * @returns feature value
   */
  private updateFeatureMap;
  /**
   * This API exposes configured overrides for features
   * @example <caption>, PCore.getEnvironmentInfo().getFeatureMap() will return feature override object</caption>
   * @returns default is empty object
   * @ignore
   */
  getFeatureMap(): import("./types").Features;
  /**
   * This API is to store keyMapping for launchpad
   * @example <caption>PCore.getEnvironmentInfo().setKeyMapping({"pxCreateDateTime" : "CreateDateTime"})</caption>
   * @private
   */
  setKeyMapping(keyMap: any): any;
  /**
   * This API gets the corresponding launchpad identifier for the specified constellation identifier.
   * If corresponding launchpad identifier is missing it will return null
   * @param keyValue - Specific constellation identifier for which we obtain corresponding launchpad identifier
   * @example <caption>PCore.getEnvironmentInfo().getKeyMapping("pzInsKey") (i.e. mapping of Constellation identifier with launchpad identifier ex. pzInskey to ID)</caption>
   * @returns Launchpad identifier if available else null
   */
  getKeyMapping(keyValue: string): string | null;
  setCoexistenceMeta(data: any): {
    appType: string;
    isCoexistence: boolean;
  };
  getCoexistenceMeta(): {
    appType: string;
    isCoexistence: boolean;
  };
  isCoexistenceAppOfType(type: string): boolean;
  /**
   * Returns list of cases added in application cases and data
   * @example <caption>PCore.getEnvironmentInfo().getCaseTypeList()</caption>
   * const caselist = PCore.getEnvironmentInfo().getCaseTypeList();
   *
   * @returns theme
   * @ignore
   */
  getCaseTypeList(): ({
    [key: string]: string;
  } & {
    hasCreateAccess?: boolean;
  })[];
  /**
   * Returns userSettings
   *
   * @example <caption>PCore.getEnvironmentInfo().getUserSettings</caption>
   * const userSettings = PCore.getEnvironmentInfo().getUserSettings();
   * @returns userSettings
   * @ignore
   */
  getUserSettings(): string | null;
  getAvailableLocalePacks(): string[];
  /**
   * Returns performance dashboard access defined on currently rendered portal.
   *
   * @example <caption>PCore.getEnvironmentInfo().getPerformanceDashboardVisibility()</caption>
   * const isPerformanceDashboardVisible = PCore.getEnvironmentInfo().getPerformanceDashboardVisibility();
   *
   * @returns {boolean} Performance dashboard visibility
   * @private
   */
  getPerformanceDashboardVisibility(): boolean;
  /**
   * Sets performance dashboard access defined for current portal.
   * @param {boolean} isPerformanceDashboardVisible - performance dashboard visibility
   *
   * @example <caption>PCore.getEnvironmentInfo().setPerformanceDashboardVisibility(true)</caption>
   * PCore.getEnvironmentInfo().setPerformanceDashboardVisibility(true);
   *
   * @returns {undefined}
   * @private
   */
  setPerformanceDashboardVisibility(isPerformanceDashboardVisible: boolean): void;
  /**
   * Returns flag to make extra call for loading default home
   *
   * @example <caption>PCore.getEnvironmentInfo().getIsExtraCallRequiredForDefaultHome()</caption>
   * const isExtraCallRequiredForDefaultHome = PCore.getEnvironmentInfo().getIsExtraCallRequiredForDefaultHome();
   *
   * @returns {string} flag to make extra call for loading default home
   * @private
   */
  getIsExtraCallRequiredForDefaultHome(): string;
  /**
   * Sets if extra call is required to load default home data.
   * @param {boolean} isExtraCallRequiredForDefaultHome - flag to make extra call for loading default home
   *
   * @example <caption>PCore.getEnvironmentInfo().setIsExtraCallRequiredForDefaultHome(true)</caption>
   * PCore.getEnvironmentInfo().setIsExtraCallRequiredForDefaultHome("true");
   *
   * @returns {undefined}
   * @private
   */
  setIsExtraCallRequiredForDefaultHome(isExtraCallRequiredForDefaultHome: string): void;
  /**
   * Returns Authentication timeout value configured on access group
   *
   * @example <caption>PCore.getEnvironmentInfo().getInactivityTimeout()</caption>
   * const authenticationTimeout = PCore.getEnvironmentInfo().getInactivityTimeout();
   *
   * @returns {Number|undefined} Authentication timeout value
   * @private
   */
  getInactivityTimeout(): number | undefined;
  /**
   * This API exposes isUnifiedObject flag present in environment-info of launchpad
   * @example <caption> PCore.getEnvironmentInfo().isUnifiedObject() will return isUnifiedObject flag </caption>
   * @returns default false
   * @private
   */
  isUnifiedObject(): boolean;
  getRuntime(): string;
}

export declare class Debugger {
  isXRayOn(): boolean;
  toggleXRay(xray: boolean): void;
  onXRay(callback: () => void): void;
}

export declare class AuthUtils {
  getAuthInstance(config: PegaAuthConfig): PegaAuth;
  revokeTokens(): Promise<void>;
  setAuthorizationHeader(value: string): void;
  setTokens(tokenInfo: TokenInfo): void;
}

export declare class PegaAuthConfig {
  clientId: string;
  clientSecret: string;
  endPoints: {
    token: string;
    revoke: string;
    authorize: string;
  }
}

export declare class PegaAuth {
  authMessageHandler(): any;
  config: any;
  getRedirectUriOrigin(): string;
  getTokenAndFinish(): void;
  ssKeyConfig: PegaAuthConfig;
}

export interface TokenInfo {
  access_token: string;
  refresh_token?: string;
  session_index?: string;
  expires_in: number;
  token_type: string;
}

export declare class PubSubUtils {
  cleanContextSubscribers(contextName: string): void;
  publish(eventType: string, payload: any): void;
  subscribe(eventType: string, subscriptionItem: () => void, subscriptionItemName: string, subscribeOnce: boolean, contextName: string): void;
  subscribeOnce(eventType: string, subscriptionItem: () => void, subscriptionItemName: string): void;
  unsubscribe(eventType: string, subscriptionItemName: string, contextName: string): void;
}

export declare class Events {
  getCaseEvent(): { [key: string]: string };
  getDataEvent(): { [key: string]: string };
}

export declare class AttachmentUtils {
  cancelRequest(fileID: string): void;
  deleteAttachment(attachmentID: string, context: string): Promise<void>;
  downloadAttachment(attachmentID: string, context: string, responseEncoding?: string): Promise<{ data: string }>;
  editAttachment(attachmentID: string, attachmentMetaData: Map<string, any>, context: string): Promise<object>;
  getAttachmentCategories(caseID: string, type: string, context: string): Promise<object>;
  getCaseAttachments(caseID: string, context: string, includeThumbnail?: boolean): Promise<object>;
  linkAttachmentsToCase(caseID: string, attachments: any[], attachmentType: string, context: string): Promise<object>;
  uploadAttachment(
    file: File,
    onProgess?: (event: ProgressEvent) => void,
    onError?: (fn: (error: Error) => boolean, file: File) => void,
    contextName?: string
  ): Promise<FileRef>;
}

export interface FileRef {
  ID: string;
}

export declare class MessageManager {
  clearMessages(config: MessageConfig): void;
  addMessages(config: MessageConfig): void;
}

export interface MessageConfig {
  /** type of the message e.g. error or info */
  type?: string;
  /** category type of the messages e.g. property, PAGE or HTTP */
  category?: string;
  /** context where messages gets displayed e.g. app or app/primary_2 or app/primary_2/workarea_1 */
  context?: string | null;
  /** property name e.g. firstName */
  property?: string;
  /** page reference of the property */
  pageReference?: string;
  isListEntry?: boolean;
  clearContextMessages?: boolean;
  stateObj?: any;
  messages?: Message[];
  deletingIndex?: number;
  fieldLabel?: string;
  message?: string;
}

export interface Message {
  type?: string;
  message?: string;
}

export declare class MashupApi {
  createCase(
    caseTypeId: string,
    targetContext: string,
    options: CreateCaseOptions
  ): Promise<void>;
  openAssignment(assignmentId: string, targetContext: string): Promise<void>;
  openPage(pageName: string, className: string, targetContext: string): Promise<void>;
}

export interface CreateCaseOptions {
  pageName: string;
  startingFields: Object;
}

export interface PConnectConfig {
  meta: ItemView;
  options: PConnectConfigOptions;
}

export interface PConnectConfigOptions {
  context: string;
  pageReference?: string;
  containerName?: string;
  containerItemName?: string;
  parentPageReference?: string;
  hasForm?: boolean;
  isFlowContainer?: boolean;
  referenceList?: string;
}

export declare class ContainerItems {
  addContainerItems(items: string[]): void;
}

export declare class PStore<T> {
  getState(): T;
  dispatch(_: any): any;
  replaceReducer(nextReducer: any): void;
  subscribe(subscription: () => void): () => void;
}

export declare class DataApiUtils {
  getData<T = any>(
    dataViewName: string,
    payload: Object,
    context?: string
  ): Promise<DataHttpResponse<T>>;
}

export declare class ContainerUtils {
  getActiveContainerItemName(context: string): string;
  getContainerAPI(): ContainerAPI;
  getContainerItemData(target: string, itemId: string): ContainerItem;
  hasContainerItems(target: string): boolean;
}

export declare class ContainerAPI {
  addContainerItems(items: string[]): void;
  addContainerItem(container: ContainerItem): void;
}

export interface PCoreConstants {
  MODAL: string;
  PAGE: string;
  CREATE_DETAILS_VIEW_NAME: string;
  CASE_INFO: {
    CASE_TYPE_ID: string;
    CASE_INFO_ACTIONS: string;
    ACTIVE_ACTION_ID: string;
  };
  APP: {
    APP: string;
  };
  CONTAINER_TYPE: {
    MULTIPLE: string;
  };
  MESSAGES: {
    MESSAGES_TYPE_ERROR: string;
  };
  PUB_SUB_EVENTS: {
    EVENT_CANCEL: string;
    CASE_EVENTS: {
      CREATE_STAGE_SAVED: string;
    }
  };
}

export interface DataHttpResponse<T> {
  data: DataResponse<T>;
}

export interface DataResponse<T> {
  data: T[];
  resultCount: number;
  fetchDateTime: string;
  pageNumber: number;
  pageSize: number;
}
